;***************************************************************************
;
; File Name		:"DSPDrv.asm"
; Title			:DSP communication Driver for MAGOR CP01-1v2 Chipset
; Date			:2002.12.16.
; Date (Last)		:2002.12.16.
; Version		:1.00
; Support telephone	:+36-70-333-4034,  Old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AT90S8515
;
;***************************************************************************
;	D E S C R I P T I O N
;
;  - Communication to MAGOR CP01-1v2 MACH Chipset
;
;  Adat olvasas a portrol: 1. kiolvassuk a Controll Registert
;			   2. kiolvassud a data registert, ekkor INT keletkezik
;			      a DSP fele, ugyanakkor a sajatunkt toroldik
;
;  Adat irasa a portra: 1. Elleorizni, hogy szabad-e a port
;			2.
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  why
;	----	----------  ---	--------------------------------------------
;	1.00	2002.12.16  VFX	Creation
;
;
;**************************************************************************
;* Hardware Abstraction Layer

;INT1 Interrupt line used


;0x3F01 SWSTATA
;	 0. -				0. -
;	 1. -				1. -
;	 2. -				2. -
;	 3. -				3. -
;	 4. -				4. -
;	 5. -				5. DSP Rx Full
;	 6. -				6. DSP Tx Full
;	 7. DSP TxReg ures flag torlese	7. DSP Tx Empty

;0x3F02 DSPDATA				DSPDATA / IOPDAT
;0x3F03 DSPADDCMDA			ICADSPCMD


;***************************************************************************
;* Const Def

.equ	DSPTxEmpty = 7
.equ	DSPTxFull  = 6
.equ	DSPRxFull  = 5


.equ	MSGRx_Buffer_Length = 16
.equ	MSGTx_Buffer_Length = 12


;***************************************************************************
.DSEG

MSGRx_Head:	.byte 1
MSGRx_Tail:	.byte 1
MSGRx_Data:	.byte MSGRx_Buffer_Length

MSGTx_Head:	.byte 1
MSGTx_Tail:	.byte 1
MSGTx_Data:	.byte MSGTx_Buffer_Length


.CSEG
;***************************************************************************
; INT1 line & variable init
;
Init_INT1:

         	CBI	DDRD,3		;INT1 pin Input & pull-up
		SBI	PORTD,3

		clr	R16
		sts	MSGTx_Head,R16
		sts	MSGTx_Tail,R16
		sts	MSGRx_Head,R16
		sts	MSGRx_Tail,R16


		ldi	R16,(1<<DSPTxEmpty)
		sts	SWSTATA,R16		;DSP Tx int torlese,
						;mert meg nem kuldtunk semmit

		in	R16,MCUCR
		cbr	R16,0b00001100		;INT1 now low-level trigered
		OUT	MCUCR,R16		;3:2 = Inerrupt Sense Control for INT1
						; 00 -> Low Level
						; 01 -> Reserved
						; 10 -> Falling Edge
						; 11 -> Rising Edge

		ldi	R16,128			;INT1 IRQ flag torlese
		out	GIFR,R16

		in	R16,GIMSK
		ori	R16,128
		OUT	GIMSK,R16		;7. = 1 EXT INT1 ACTIVE
		ret


;************************************************************************
;** Calc Buffer Free Space
; In:	R16 - Buffer Length
;       R2  - Head
;	ZL  - Tail
;
; Out:	R2 - Head
;	R16 - Free Space in Buffer
;       Flag z = 1 akkor ures
;
; Alt:  ZH,ZL
;
FreeMSGTx:	ldi	R16,MSGTx_Buffer_Length
		lds	R2,MSGTx_Head
		lds	ZL,MSGTx_Tail
FreeInBuff:
		sub	ZL,R2
		breq	FreeIn1		;T=H, Free= Tx_Buffer_Length
		brcs	FreeIn2
                clr	R16
FreeIn2:	add	R16,ZL
		dec	R16
FreeIn1:	ret


;*****************
; Megprobal uzenetet elhelyezni a pufferben [R1:R0 az uzenet]
; c=1 ha nem ment
;
TryPutMsg:
		rcall	FreeMSGTx			;R16=free space
		cpi	R16,2
		brsh	OkVanHely
		ret

OkVanHely:	ldi	ZL,low(MSGTx_Data)
		ldi	ZH,High(MSGTx_Data)
		mov	R16,R2
		clr	R2
		add	ZL,R16
		adc	ZH,R2

		st	Z+,R0				;data eltarolva
		inc	R16
		cpi	R16,MSGTx_Buffer_Length
		brcs	OkKovetkezoByte
		clr	R16
		ldi	ZL,low(MSGTx_Data)
		ldi	ZH,high(MSGTx_Data)

OkKovetkezoByte:
		st	Z,R1				;parancs eltarolva
		inc	R16
		cpi	R16,MSGTx_Buffer_Length
		brcs	OkFinal
		clr	R16

OkFinal:	sts	MSGTx_Head,R16

		cli
		rcall	EXT_INT1			;a RETI feloldja a megszkitast tiltast
		clc
		ret


EXT_INT1:
		push	ZL
		push	ZH
		in	ZL,SREG				; preserve main OS status reg.
		push	ZL
		push	R16
		push	R2


		lds	R16,SWSTATA			;Nezzuk miert is volt megszakitas
		sbrs	R16,DSPRxFull			;Vettunk valamit
		 rjmp	Txlesz
		
							;Itt vettunk valamit

		lds	R16,DSPADDCMDA			;Comand/Controll reg.
		lds	R16,DSPDATA			;DSP Data Reg.
							;most csak elengedjuk a levegobe

Txlesz:
		lds	R16,SWSTATA			;Tx reg ures?
		sbrs	R16,DSPTxFull			;Ha Tx reg teli
		 rjmp	Txures

							;Itt meg teli van az ado
							;ha ures lesz INT generalodik
		rjmp	INT1End

Txures:
		ldi	R16,(1<<DSPTxEmpty)
		sts	SWSTATA,R16			;DSP Tx Reg. Int. torlese

		lds	ZL,MSGTx_Head
		lds	R16,MSGTx_Tail
		cp	ZL,R16
		brne	VanBenneKuldendo

		rjmp	INT1End

VanBenneKuldendo:


		ldi	ZL,low(MSGTx_Data)
		ldi	ZH,High(MSGTx_Data)
		clr	R2
		add	ZL,R16
		adc	ZH,R2

		ld	R2,Z+
		inc	R16
		sts	DSPDATA,R2


		cpi	R16,MSGTx_Buffer_Length
		brcs	kovb1
		clr	R16
		ldi	ZL,low(MSGTx_Data)
		ldi	ZH,high(MSGTx_Data)

kovb1:		ld	R2,Z
		inc	R16
		cpi	R16,MSGTx_Buffer_Length
		brcs	kovb2
		clr	R16
kovb2:		sts	MSGTx_Tail,R16
		sts	DSPADDCMDA,R2			;DSP fele elkuldve!!


INT1End:
		pop	R2
		pop	R16
		pop	ZL
		out	SREG,ZL				; restore previous status reg
		pop	ZH
		pop	ZL
		reti

