;*
;* Title:		Accessing the EEPROM
;* Version:		2.0
;* Last updated:	98.06.15
;* Target:		AT90Sxxxx (All AVR Devices)
;*
;* Support E-mail:	avr@atmel.com
;*
;***************************************************************************

	
;***************************************************************************
;* 
;* EEWrite
;*
;* This subroutine waits until the EEPROM is ready to be programmed, then
;* programs the EEPROM with register R0 at address ZL:ZH
;*
;* Number of words	:1200 ; 5 + return
;*			:8515 ; 7 + return
;* Number of cycles	:1200 ; 8 + return (if EEPROM is ready)
;*			:8515 ; 11 + return (if EEPROM is ready)
;* Low Registers used	:R0
;* High Registers used:	;ZL,ZH
;*
;***************************************************************************
EEWrite:
	sbic	EECR,EEWE	;if EEWE not clear
	rjmp	EEWrite		;    wait more

	out 	EEARH,ZH	;output address high for 8515
	out	EEARL,ZL	;output address low for 8515
		

	out	EEDR,R0		;output data
	sbi 	EECR,EEMWE	;set master write enable, remove if 1200 is used	
	sbi	EECR,EEWE	;set EEPROM Write strobe
				;This instruction takes 4 clock cycles since
				;it halts the CPU for two clock cycles
	ret

;***************************************************************************
;* 
;* EERead
;*
;* This subroutine waits until the EEPROM is ready to be programmed, then
;* reads the register R0 from address ZL:ZH
;*
;* Number of words	:1200 ; 5 + return
;*			:8515 ; 6 + return
;* Number of cycles	:1200 ; 8 + return (if EEPROM is ready)
;*			:8515 ; 9 + return (if EEPROM is ready)
;* Low Registers used	:R0
;* High Registers used:	:ZL,ZH
;*
;***************************************************************************
EERead:
	sbic	EECR,EEWE	;if EEWE not clear
	rjmp	EERead		;    wait more
	out 	EEARH,ZH	;output address high for 8515
	out	EEARL,ZL	;output address low for 8515
	sbi	EECR,EERE	;set EEPROM Read strobe
				;This instruction takes 4 clock cycles since
				;it halts the CPU for two clock cycles
	in	R0,EEDR		;get data
	ret

		
;***************************************************************************
;* 
;* EEWrite_seq
;*
;* This subroutine waits until the  EEPROM is ready for programming,
;* write R0 to EEPROM and increments the EEPROM address by one
;*
;* Number of words	:1200 ; 7 + return
;*			:8515 ; 10 + return
;* Number of cycles	:1200 ; 10 + return (if EEPROM is ready)
;*			:8515 ; 15 + return (if EEPROM is ready)
;* Low Registers used	:R0
;* High Registers used:	:ZL,ZH
;*
;***************************************************************************
EEWrite_seq:
	sbic	EECR,EEWE	;if EEWE not clear
	rjmp	EEWrite_seq	;   wait more

	out	EEDR,R0		;output data
	sbi 	EECR,EEMWE	;set master write enable, remove if 1200 is used	
	sbi	EECR,EEWE	;set EEPROM Write strobe
				;This instruction takes 4 clock cycles since
				;it halts the CPU for two clock cycles
	in	ZL,EEARL	;get address low 8515
	in	ZH,EEARH	;get address high 8515	
 	adiw	ZL,1		;increment address 8515
	out	EEARL,ZL	;output address 8515
	out	EEARH,ZH	;output address 8515
	ret

;***************************************************************************
;* 
;* EERead_seq
;*
;* This subroutine increments the address stored in EEAR and reads the 
;* EEPROM into the register variable "EEdrd_s".

;* Number of words	:1200 ; 5 + return
;*			:8515 ; 7 + return
;* Number of cycles	:1200 ; 8 + return (if EEPROM is ready)
;*			:8515 ; 11 + return (if EEPROM is ready)
;* Low Registers used	:R0 
;* High Registers used:	:Zl,ZH
;*
;***************************************************************************
EERead_seq:
	sbic	EECR,EEWE	;if EEWE not clear
	rjmp	EERead_seq	;   wait more

	sbi	EECR,EERE	;set EEPROM Read strobe
				;This instruction takes 4 clock cycles since
				;it halts the CPU for two clock cycles
	in	R0,EEDR		;get data
	in	ZL,EEARL	;get address low 8515
	in	ZH,EEARH	;get address high 8515	
	adiw	ZL,1		;increment address 8515
	out	EEARL,ZL	;output address 8515
	out	EEARH,ZH	;output address 8515
	ret

