;***************************************************************************
;
; File Name		:"KeybDec.asm"
; Title			:Keyboard ScanCode Decoder for MAGOR CP01-1v2
; Date			:2002.12.13.
; Date (Last)		:2002.12.13.
; Version		:1.05
; Support telephone	:+36-70-333-4034,  Old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AT90S8515
;
;***************************************************************************
;	D E S C R I P T I O N
;
;  - Scan code decoder for MARGOR CP01-1v2 (used PS2Drv.asm)
;
; This program uses ScanCode 2 Table of Keyboard for decoding
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who		why
;	----	----------  ---		--------------------------------------------
;	1.00	2002.12.13  VFX		Creation
;	1.05	2002.12.15  VFX		Remove Scancode -> ASCII, Scancode -> VirtualKey added
;
;**************************************************************************
;* Hardware Abstraction Layer


;***************************************************************************
;* Const Def

.include "keyboard.inc"

.EQU	Scan_Break = 0x01	;VK Brake kod

;***************************************************************************
;**** VARIABLES
.DSEG

Kb_scanFlg:	.byte 1		;Extended & Break kod figyeleshez kell
Virt_KeyStat:	.byte 1		;Dekodolt Virtual Key & Status


;***************************************************************************
.ESEG


;***************************************************************************
.CSEG

;***************************************************************************
;*  Init_ScanCodeDecoder
;*
;*  Inicializalja a ScanCode Dekodert
;
Init_ScanCodeDecoder:
		clr	R16
		sts	Virt_KeyStat,R16
		sts	Kb_scanFlg,R16
		ret


;**************************************************************************
;* DecodeScanC
;*
;* Translate scan code to virtual key code
;
; Ha c=0, akkor R0=VirtualKey code
;		R1=VirtualKey Status Info
;    c=1, nincs mit dekodolni


DecodeScan:					;VIGYAZAT nem az a belepesi cim
						;HANEM a  "DecodeScanC" !!!!
		sts	Kb_scanFlg,R17

DecodeScanC:	rcall	Get_ScanC
		brcc	DecScn1			;ha nincs mit dekodolni kilep
		ret
DecScn1:
		mov	R16,R0			;R16-ban a vett kod
		lds	R17,Kb_scanFlg
		cpi	R17,128
		brsh	AzaszemetPauseGomb
		cpi	R17,1
		breq	DecS1
		brsh	DecS2

DecSCNormal:
		cpi	R16,0xE1		;ha igen akkor csak a Pause lehet
		brne	DecS0
		ldi	R17,134
		rjmp	DecodeScan

DecS0:
                cpi	R16,0xE0		;Extended char?
		brne	DecS1
		ldi	R17,1			;R17=1 most igy mar csak Break kod johet vagy sima kod
		rjmp	DecodeScan

DecS1:
		cpi	R16,0xF0		;Break kod?
		brne	DecS2
		ori	R17,2			;R17=2 vagy 3, igy mar csak karakter kod johet
		ldi	R16,Scan_Break
		sts	Virt_KeyStat,R16		;Break Bit beallitasa a status-ban
		rjmp	DecodeScan

DecS2:						;R17 = 3 Extended + Break + scan
						;R17 = 2 Break + Scan
						;R17 = 1 Extended + Scan
						;R17 = 0 Scan
		ldi	ZL,Low(ScanTNor*2)
		ldi	ZH,High(ScanTNor*2)
		sbrs	R17,0			;nem ugrunk, ha extended tabla kell
		 rjmp	ScDec1
		ldi	ZL,Low(ScanTExt*2)
		ldi	ZH,High(ScanTExt*2)
						;kereses a tablaban talalatig vagy tabla vegeig
ScDec1:		lpm
		adiw	ZL,1
		tst	R0			;nulla
		breq	Tablavege
		cp	R16,R0
		breq	OpsTalalt		;meg van a scan kod
		adiw	ZL,1
		rjmp	ScDec1

Tablavege:
		rcall	Init_ScanCodeDecoder
		sec				;nincs karakter jelzes
		ret

OpsTalalt:	lpm				;R0-ban a Virtual Key Code
		lds	R1,Virt_KeyStat		;R1-ben a Virtual Key Status
vegrevege:
		rcall	Init_ScanCodeDecoder	;Alaphelyzetbe allitas a kovetkezo dekodolashoz
		clc				;nincs hiba, minden ok
		ret

AzaszemetPauseGomb:
		dec	R17
		cpi	R17,128
		brne	DecodeScan
		ldi	R16,VK_PAUSE
		mov	R0,R16
		clr	R1
		rjmp	vegrevege

;SecPause:	.db	0xE1,0x14,0x77,0xE1,0xF0,0x14,0xF0,0x77		;Egyenlore nem hasznaljuk

;****************************************************************************
;* Scan kod tabla Scan2 kodokkal
;*
;** Normal scan (1 byte xx)
ScanTNor:	.db	0x4d,VK_P	;0. P
		.db	0x4b,VK_L	;1. L
		.db	0x24,VK_E	;2. E
		.db	0x1C,VK_A	;3. A
		.db	0x32,VK_B	;4. B
		.db	0x0E,'%'	;5. ~	   	%	!!!!!!!!!!!!!!!
		.db	0x29,VK_SPACE	;6. Space Bar
		.db	0x3D,VK_7	;7. 7
		.db	0x83,VK_F7	;9. F7		Menu Ok
		.db	0x79,VK_ADD	;10. NUMERIC+	+C
		.db	0x7D,VK_NUMPAD9	;11. NUMERIC9	+Z
		.db	0x75,VK_NUMPAD8	;12. NUMERIC8	+Y
		.db	0x6C,VK_NUMPAD7	;13. NUMERIC7	Used
		.db	0x46,VK_9	;14. 9
		.db	0x3E,VK_8	;15. 8
		.db	0x23,VK_D	;16. D
		.db	0x33,VK_H	;17. H
		.db	0x2D,VK_R	;18. R
		.db	0x43,VK_I	;19. I
		.db	0x3B,VK_J	;20. J
		.db	0x42,VK_K	;21. K
		.db	0x44,VK_O	;22. O
		.db	0x25,VK_4	;23. 4
		.db	0x01,VK_F9	;24. F9		Menu
		.db	0x0B,VK_F6	;25. F6		Menu Cancel
		.db	0x71,VK_SEPARATOR;26. NUMERIC. 	Used
		.db	0x74,VK_NUMPAD6	;27. NUMERIC6	+X
		.db	0x11,VK_LMENU	;28. LEFT ALT	Gyorsmenet
		.db	0x6B,VK_NUMPAD4	;29. NUMERIC4	-X
		.db	0x36,VK_6	;30. 6
		.db	0x2E,VK_5	;31. 5
		.db	0x31,VK_N	;32. N
		.db	0x34,VK_G	;33. G
		.db	0x3A,VK_M	;34. M
		.db	0x2B,VK_F	;35. F
		.db	0x1B,VK_S	;36. S
		.db	0x2C,VK_T	;37. T
		.db	0x15,VK_Q	;38. Q
		.db	0x16,VK_1	;39. 1
		.db	0x0D,VK_TAB	;41. TAB   	MODE
		.db	0x7B,0		;42. NUMERIC-	-C		!!!!!!!!!!!!!!!!!1
		.db	0x7A,VK_NUMPAD3	;43. NUMERIC3	Used
		.db	0x72,VK_NUMPAD2	;44. NUMERIC2	-Y
		.db	0x69,VK_NUMPAD1	;45. NUMERIC1	-Z
		.db	0x26,VK_3	;46. 3
		.db	0x1E,VK_2	;47. 2
		.db	0x22,VK_X	;48. X
		.db	0x35,VK_Y	;49. Y
		.db	0x1A,VK_Z	;50. Z
		.db	0x3C,VK_U	;51. U
		.db	0x2A,VK_V	;52. V
		.db	0x1D,VK_W	;53. W
		.db	0x21,VK_C	;54. C
		.db	0x49,VK_DECIMAL	;55. .
		.db	0x07,VK_F12	;59. F12	Rotate Right
		.db	0x76,VK_ESCAPE	;60. ESC	Rotate stop
		.db	0x09,VK_F10	;61. F10	Rotate Left
		.db	0x45,VK_0	;62. 0
		.db	0x4E,VK_SUBTRACT;63. -
		.db	0x05,VK_F1	;64. F1		S1
		.db	0x06,VK_F2	;65. F2		S2
		.db	0x04,VK_F3	;66. F3		S3
		.db	0x0C,VK_F4	;67. F4		S4
		.db	0x03,VK_F5	;68. F5		S5
		.db	0x5A,VK_RETURN	;75. ENTER
		.db	0x66,VK_BACK	;78. Backspace
		.dw	0x0000	;tabla vege


;******************************************************************************
; Extended scan codes (2 bytes: 0xE0, xxxx)
ScanTExt:	.db	0x7D,VK_PRIOR	;8.  PAGE UP	Menu Up
		.db	0x7A,VK_NEXT	;40. PAGE DOWN	Menu Down
		.db	0x69,VK_END	;56. End
		.db	0x75,VK_UP	;57. Up Arrow
		.db	0x6C,VK_HOME	;58. Home
		.db	0x10,VK_OEM_E4	;69. START
		.db	0x08,VK_OEM_E3	;70. STOP
		.db	0x71,VK_DELETE	;71. DEL
		.db	0x74,VK_RIGHT	;72. R Arrow
		.db	0x72,VK_DOWN	;73. Dn Arrow
		.db	0x6B,VK_LEFT	;74. L Arrow
		.db	0x70,VK_INSERT	;79. INS
		.db	0x18,0xA0	;80. Yac-16[0]
		.db	0x20,0xA1	;81. Yac-16[1]
		.db	0x28,0xA2	;82. Yac-16[2]
		.db	0x30,0xA3	;83. Yac-16[3]
		.db	0x38,0xA4	;84. Yac-16[4]
		.db	0x40,0xA5	;85. Yac-16[5]
		.db	0x48,0xA6	;86. Yac-16[6]
		.db	0x50,0xA7	;87. Yac-16[7]
		.db	0x57,0xA8	;88. Yac-16[8]
		.db	0x6F,0xA9	;89. Yac-16[9]
		.db	0x13,0xAA	;90. Yac-16[10]
		.db	0x19,0xAB	;91. Yac-16[11]
		.db	0x39,0xAC	;92. Yac-16[12]
		.db	0x53,0xAD	;93. Yac-16[13]
		.db	0x5C,0xAE	;94. Yac-16[14]
		.db	0x5F,0xAF	;95. Yac-16[15]
		.db	0x61,0xB0	;96. Yac-11[0]
		.db	0x1A,0xB1	;97. Yac-11[1]
		.db	0x22,0xB2	;98. Yac-11[2]
		.db	0x21,0xB3	;99. Yac-11[3]
		.db	0x2A,0xB4	;100. Yac-11[4]
		.db	0x32,0xB5	;101. Yac-11[5]
		.db	0x31,0xB6	;102. Yac-11[6]
		.db	0x3A,0xB7	;103. Yac-11[7]
		.db	0x41,0xB8	;104. Yac-11[8]
		.db	0x49,0xB9	;105. Yac-11[9]
		.db	0x4A,0xBA	;106. Yac-11[10]
		.db	0x58,VK_OEM_E9	;112. Yac-12[0]
		.db	0x1C,VK_OEM_EA	;113. Yac-12[1]
		.db	0x1B,VK_OEM_EB	;114. Yac-12[2]
		.db	0x23,VK_OEM_EC	;115. Yac-12[3]
		.db	0x2B,VK_OEM_ED	;116. Yac-12[4]
		.db	0x34,VK_OEM_EE	;117. Yac-12[5]
		.db	0x33,VK_OEM_EF	;118. Yac-12[6]
		.db	0x3B,VK_OEM_F0	;119. Yac-12[7]
		.db	0x42,VK_OEM_F1	;120. Yac-12[8]
		.db	0x4B,VK_OEM_F2	;121. Yac-12[9]
		.db	0x4C,VK_OEM_F3	;122. Yac-12[10]
		.db	0x52,VK_OEM_F4	;123. Yac-12[11]
		.dw	0x0000		;tabla vege

