;***************************************************************************
;
; File Name		:'Main.asm"
; Title			:CP-01-1v2 DSP Panelen a 5V-os AVR (IC21)
; Date			:2001.12.24.
; Version		:1.2.0
; Support telephone	:+36-70-333-4034,  old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AT90S8515
;
;***************************************************************************
;	D E S C R I P T I O N
;
; MAGOR CP-01-1v2  DSP Panel UART Controller & Terminal
;                  PS/2 Keyboard Controller
;		   Real Time Clock Controler
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  why
;	----	----------  ---	--------------------------------------------
;	0.01	2001.12.24  VFX	Creation
;
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	Kvarc: f=7.3728 MHz (T=135.63368 ns)
;*
;***************************************************************************


.include "8515def.inc"


;***************************************************************************
;* Const Def

.equ	SYSCLK = 7372800	;Rendszer orajel
.equ	TRUE	=0FFh		;Yes
.equ	FALSE	=00h		;No

.EQU	LEDPNormal = 25		;LED villogasi sebessege 4db/s

;**************************************************************************
;* Hardware Def.
;
; AVR IC21 regiszterei AVRA
;


; OC1A - LCD haterfeny ki/be X14-en (PD5)
; OC1B - Analog PWM kimenet az X16-on, vagy digitalis PWM a Zummernek
; INT1 - MACH kozositett interrupt bemenet Aktiv: Low Level
; PD4  - Dallas 1-wire kulcsolvaso vezetek



.EQU	LCDBackLightPort = PORTD
.EQU	LCDBackLightDIR = DDRD
.EQU	LCDBackLight = PD5


;***************************************************************
;** AVR IC21 regiszterei AVRA a M4A3-256/128-ban

; 	WRITE				READ
;
;0x3F00 ICACMD				STATA
;	 0. HBR allapota (=1 -> HBR=0)	0. HBG allapota
;	 1. -				1. RESET allapota
;	 2. -				2. AVRA Veszkor
;	 3. -				3. DSP Veszkor
;	 4. - 				4. 24V input allapota
;	 5. -				5. -
;	 6. -				6. -
;	 7. -				7. -

;0x3F01 SWSTATA
;	 0. -				0. -
;	 1. -				1. -
;	 2. -				2. -
;	 3. -				3. -
;	 4. -				4. -
;	 5. -				5. DSP Rx Full
;	 6. -				6. DSP Tx Full
;	 7. DSP TxReg ures flag torlese	7. DSP Tx Empty

;0x3F02 DSPDATA				DSPDATA / IOPDAT
;0x3F03 DSPADDCMDA			ICADSPCMD

;0x3F04 -				SerialNumLL
;0x3F05 -				SerialNumLH
;0x3F06 -				SerialNumHL
;0x3F07 -				SerialNumHH
;0x3F08 RESET port			-
;0x3F09 Veszkor allapota		-
;0x3F0A SWINT				-
;	 0. bit =1 SW INT1



;Registers in MACH, memorymapped

.EQU	ICACMD  = 0x3F00
.EQU	SWSTATA = 0x3F01
.EQU	DSPDATA = 0x3F02
.EQU	DSPADDCMDA = 0x3F03
.EQU	RegRES = 0x3F08
.EQU	RegVEsz = 0x3F09
.EQU	SWINT = 0x3F0A

.EQU	STATA = 0x3F00
.EQU	ICADSPCMD = 0x3F03
.EQU	SerialNumLL = 0x3F04		;Serial Number 4 byte
.EQU	SerialNumLH = 0x3F05
.EQU	SerialNumHL = 0x3F06
.EQU	SerialNumHH = 0x3F07



;Event definitions
.equ	Ev_Keybd	= 0b11110000		;keyboard esemeny 16 alesemy lehet







.EQU	LEDDIR	= DDRB
.EQU	LEDPORT	= PORTB
.EQU	LED = 4					;A "Elek LED" bitcime

;********************************************************************
;Revision perversion
;
;if you want to put it in a message like this:
;VERSION: .db	"FIRMWARE VERSION ",MAJOR_REV,".","MINOR_REV","MINOR_REVB",CR,LF
;

.equ	MAJOR_REV	='2'
.equ	MINOR_REV	='0'
.equ	MINOR_REVB	='2'



;***************************************************************************
;**** VARIABLES
.DSEG

Flag1:	 .BYTE 1		;Flagbitek

SCNT1:	 .BYTE 2		;16 bit Software Timer1 /LED villogas periodusideje
SCNT2:	 .BYTE 2		;16 bit Software Timer2 /XMODEM Timeout
SWTmr0:	 .BYTE 1		;8 bites Software Timer / keyboard Time out counter

LEDPer:	 .byte 2		;LED villogasi periodusa 0.01s egysegekben



;***************************************************************************
.ESEG

dumyEEPD:	.dw 0	;ez azert mert 0. cim tapelvetelkor serulhet

sallang:	.db "(C) 2001,2002 MAGOR CP01-1v2 Terminal Interface [2001.12.24.]",CR
		.DB "Firmware version ",MAJOR_REV,".",MINOR_REV,MINOR_REVB," "
		.DB "Latest Upd.: 2002.12.18. ",CR
		.DB "MagTerm by VFX [info@vfx.hu] ",CR


		.db "and blablaballllll aaaaaaa blabla blabla blabla blabla... "

;***************************************************************************
;****		I N T E R R U P T S
;****
;***************************************************************************
.CSEG
	rjmp RESET		;Reset Handle
	rjmp EXT_INT0		;IRQ0 Handle
	rjmp EXT_INT1		;IRQ1 Handle
	rjmp TIM1_CAPT		;Timer1 capture Handle
	rjmp TIM1_COMPA		;Timer1 compare Handle
	rjmp TIM1_COMPB		;Timer1 compare Handle
	rjmp TIM1_OVF		;Timer1 overflow Handle
	rjmp TIM0_OVF		;Timer0 overflow Handle
	rjmp SPI_STC		;SPI Transfer Complete Handle
	rjmp UART_RXC		;UART RX Complete Handle
	rjmp UART_DRE		;UDR Empty Handle
	rjmp UART_TXC		;UART TX Complete Handle
	rjmp ACI_INT		;Analog Comparator Handle




.include	"RS5C313.asm"
.include	"EEprom.asm"
.include	"isr.asm"
.include	"mat32.asm"



;************************************************************************
;************************************************************************
;* RESET

RESET:		CLI			;GLOBAL INTERRUP DIS.

		ldi R16,high(RAMEND)	;setup stack
       		out SPH,R16
       		ldi R16,low(RAMEND)
       		out SPL,R16

		ldi	ZL,low(65535) 	;izombol varakozunk kell kb 35.5ms-t
       		ldi	ZH,high(65535)	;csak, hogy legyen mivel tolteni az idot
HWvar:
 		sbiw	ZL,1		;[2]
       		brne	HWvar		;[2]


		LDI	R16,0b10100010
		OUT	MCUCR,R16	;7. = 1 SRE External SRAM Enabled
					;6. = 1 SRW Ext. SRAM Wait State Enabled
		       			;5. = 1 Sleep Enable
		       			;4. = 0 SLEEP => Idle Mode
		       			;     1 SLEEP => Power Down Mode
		       			;3:2 = Inerrupt Sense Control for INT1
		       			; 00 -> Low Level INT1
		       			; 01 -> Reserved
		       			; 10 -> Falling Edge INT1
		       			; 11 -> Rising Edge INT1
		       			;1:0 = Inerrupt Sense Control for INT0
		       			; 00 -> Low Level INT0
					; 01 -> Reserved
			       		; 10 -> Falling Edge INT0
			       		; 11 -> Rising Edge INT0



	LDI	R16,0b00000000	;INT0, INT1 Disabled
	OUT	GIMSK,R16	;7. = 1 EXT INT1 ACTIVE
				;6. = 1 EXT INT0 ACTIVE



	LDI	R16,0b00000010
	OUT	TIMSK,R16 	;7. = 1 TC1 OWRF EN.
				;6. = 1 TC1 COMP MATCH EN.
				;3. = 1 TC1 INP CAPTURE EN.
				;1. = 1 TC0 OWRF EN.


	LDI	YL,LOW(RAMSTRT)              	;Ezt maj meg kell szuntetni, hogy Y szabad legyen
	LDI	YH,HIGH(RAMSTRT) ;BAZIS 'Y' +0

	LDI	R16,0
	STD	Y+Flag1-RAMSTRT,R16


	rcall	InitTimer1
	rcall	InitTimer0


;************************************************************************
;**  I/O HW SETUP

	CBI	LEDPORT,LED		;LED vilagit
	SBI	LEDDIR,LED		;LED -> Output

	ldi	R16,Low(LEDPNormal)
	sts	LEDPer+0,R16
	ldi	R16,High(LEDPNormal)
	sts	LEDPer+1,R16		;kezdo LED villogasi sebesseg

	LDS	R16,LEDPer+0
	STD	Y+SCNT1+0-RAMSTRT,R16
	LDS	R16,LEDPer+1
	STD	Y+SCNT1+1-RAMSTRT,R16


	sbi	LCDBackLightPort,LCDBackLight	;LCD-n hatterfeny bekapcsolva alapbol


;************************************************************************

					;megszkitas engedelyezese elott
					;toroljuk az INT flagaket
					;ami eddig jott volna megszakitas az torlodik

		ldi	R16,255
		out	USR,R16		;UART status torlese

		ldi	R16,192		;INT0,1 IRQ flag torlese
		out	GIFR,R16

		SEI			;SET GLOBAL INT EN.

		rcall	Init_RTC
		rcall	Init_UART

				;Copryright felirat kikuldese

		ldi	R16,Low(ECopyr*2)
		ldi	R17,High(ECopyr*2)	;0. EEPROM address not used!
		rcall	SendStrW
		nop
		nop                  		;Ez a ket nop is bug miatt kell???
		rcall	SendSerialNum


		ldi	R16,Low(ECD*2)
		ldi	R17,High(ECD*2)
		rcall	SendStrW
		rcall	Tm_GetDate

		ldi	R16,Low(ECT*2)
		ldi	R17,High(ECT*2)
		rcall	SendStrW
		rcall	Tm_GetTime


		rcall	Init_INT1		;Ezt meg a PS2 elott meg kell tenni
		rcall	Init_ScanCodeDecoder	;Ez mindig az Init_PS2 elott legyen!!
		rcall	Init_PS2
		rcall	Init_Term

		ldi	R16,CR
		rcall	SendChrW


;***********************************************************************
;***********************************************************************
;********     M     A      I      N				********
;********							********
;***********************************************************************
Main:		SLEEP
		nop

M2:

		rcall	Terminal

		rcall	DecodeScanC
		brcs	Nokeyhere		;hiba volt nincs billentyu

		ldi	R16,Ev_Keybd
		or	R1,R16
NemMentEl:

		rcall	TryPutMsg		;berakjuk az MSG pufferbe
						;INT1 meg majd elkuldi
		brcs	NemMentEl


Nokeyhere:


Finish:	
		rcall	LEDFlash
		rjmp	Main


;************************************************
; LED villogtato rutin

LEDFlash:	LDD	R16,Y+SCNT1+0-RAMSTRT
		LDD	R17,Y+SCNT1+1-RAMSTRT
		OR	R16,R17
		BREQ	LEDvil
		ret
LEDvil:		LDS	R16,LEDPer+0
		STD	Y+SCNT1+0-RAMSTRT,R16
		LDS	R16,LEDPer+1
		STD	Y+SCNT1+1-RAMSTRT,R16

TogleLED:	SBIS	LEDPORT,LED		;kihagy, ha LED=1
		 RJMP	Led_be
		CBI	LEDPORT,LED		;LED eg
		ret
Led_be:		SBI	LEDPORT,LED		;LED nem eg
		ret


.include	"DSPDrv.asm"
.include	"user.asm"
.include	"PS2Drv.asm"
.include	"KeybDec.asm"
.include	"rs232.asm"
.include	"xmodem.asm"
.include	"terminal.asm"


;***************************************************************************
;**** VARIABLES
.DSEG
Stack:		.byte 64	;Ezt fonttartjuk a stack-nek
MaxUsedRAM:	.BYTE 1		;Flagbitek
