;***************************************************************************
; 
; File Name		:"RS5C313.asm"
; Title			:RICOH RS5C313 Ultra-compact real-time clock IC 
; Date			:2002.06.12.
; Version		:1.00
; Support telephone	:+36-70-333-4034
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AT90S8515
; 
;***************************************************************************
;	D E S C R I P T I O N
;
;  - communication with RICOH RS5C313 Ultra-compact real-time clock IC 
;
;	CE = H		-> Chip enable (active H) 
;	SCLK = fall-edge-> Clock to RTC (samle falling edge)
;	SIO = 		-> Bi-Dir Serial data
;	INTR = L	-> Interrupt output (active L)
;
;
;
;  - RTC hasznalatakor a kulsor RAM elerest tiltani kell, hogy bit szinten
;    lehessen a SIO/CLK labakat hasznalni!
;
;  - CE setup/hold time 300ns
;  - Tclk		600ns
;
;
;
;
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y 
;
;
;       rev.      date      who  why
;	----	----------  ---	--------------------------------------------
;	0.01	2002.06.12  VFX	Creation
;
;
;**************************************************************************
;* Hardware Abstraction Layer

.EQU	RTC_CEPort = PORTB
.EQU	RTC_CEDIR = DDRB
.EQU	RTC_CEPIN = PINB
.EQU	RTC_CE  = 1

.EQU	RTC_SIOPort = PORTC
.EQU	RTC_SIODIR = DDRC
.EQU	RTC_SIOPIN = PINC
.EQU	RTC_SIO = 0


.EQU	RTC_CLKPort = PORTC
.EQU	RTC_CLKDIR = DDRC
.EQU	RTC_CLKPIN = PINC
.EQU	RTC_CLK = 1



;***************************************************************************
;* Const Def


;***************************************************************************
;**** VARIABLES
.DSEG


Time:		.byte	9			;Time -> 00:00:00 + Line end
Date:		.byte	11			;Date -> 2000-01-01 + Line end

;***************************************************************************
.ESEG


;***************************************************************************
.CSEG

;***************************************************************************
; RTC inicializalasa HW szinten
;
;
Init_RTC:	cbi	RTC_CEPort,RTC_CE
		sbi	RTC_CEDIR,RTC_CE	;RTC_CE output = L, TRC disabled

		cbi	RTC_SIOPort,RTC_SIO
		sbi	RTC_SIODIR,RTC_SIO	;RTC_SIO Output = L

		cbi	RTC_CLKPort,RTC_CLK
		sbi	RTC_CLKDIR,RTC_CLK	;RTC_CLK Output = L

		ldi	R16,0x0E		;Control register
		ldi	R17,0b00000110
		rcall	RTC_WriteData

		ldi	R16,0x07		;Interrupt cycle register
		ldi	R17,0b00000000		;Int. Disabled
		rcall	RTC_WriteData

		ret



;***************************************************************************
; RTC Get Time
;
; In:  -
; Out: var Time (00:00:00)
; Used: R0,R1,R16,R17,XL,XH

RTC_GetTime:	ldi	XL,Low(Time)
		ldi	XH,High(Time)

		in	R16,MCUCR
		push	R16
		andi	R16,0b01111111		;External SRAM Disabled
		out	MCUCR,R16

		sbi	RTC_CEPort,RTC_CE	;CE = H, IC active

		ldi	R17,5
		mov	R1,R17
		ldi	R17,3

RTC_GetT1:	mov	R16,R1
		dec	R1
		rcall	RTC_BurstRD
		ori	R16,0x30
		st	X+,R16
		mov	R16,R1
		dec	R1
		rcall	RTC_BurstRD
		ori	R16,0x30
		st	X+,R16
		ldi	R16,':'
		st	X+,R16
		dec	R17
		brne	RTC_GetT1

		cbi	RTC_CEPort,RTC_CE	;CE = L, IC inactive
		nop
		cbi	RTC_SIOPort,RTC_SIO
		sbi	RTC_SIODIR,RTC_SIO	;RTC_SIO Output = L

		pop	R0
		out	MCUCR,R0		;MCUCR visszaallitasa
		ret





;***************************************************************************
; RTC Set Date
;
; In:  var Date (2000-00-00)
; Out:  -
; Used: R0,R1,R16,R17,R18,XL,XH

RTC_SetDate:	ldi	XL,Low(Date+2)
		ldi	XH,High(Date+2)
		ldi	R18,0x0D		;year tens register
		rjmp	RTS_SetDT


;***************************************************************************
; RTC Set Time
;
; In:  var Time (00:00:00)
; Out:  -
; Used: R0,R1,R16,R17,R18,XL,XH

RTC_SetTime:
		ldi	R16,0x0E		;Control Register
		ldi	R17,0b00000110
		rcall	RTC_WriteData		;set 24 hours count

		ldi	XL,Low(Time)
		ldi	XH,High(Time)
		ldi	R18,5			;hour tens


RTS_SetDT:	in	R16,MCUCR
		push	R16
		andi	R16,0b01111111		;External SRAM Disabled
		out	MCUCR,R16

		sbi	RTC_CEPort,RTC_CE	;CE = H, IC active

		mov	R1,R18
		ldi	R18,3

RTC_SetT1:	mov	R16,R1
		dec	R1
		ld	R17,X+
		rcall	RTC_BurstWD
		mov	R16,R1
		dec	R1
		ld	R17,X+
		rcall	RTC_BurstWD
		ld	R17,X+
		dec	R18
		brne	RTC_SetT1

		cbi	RTC_CEPort,RTC_CE	;CE = L, IC inactive
		nop
		cbi	RTC_SIOPort,RTC_SIO
		sbi	RTC_SIODIR,RTC_SIO	;RTC_SIO Output = L

		pop	R0
		out	MCUCR,R0		;MCUCR visszaallitasa
		ret


;***************************************************************************
; RTC Get Date
;
; In:  -
; Out: var Date (2000-00-00)
; Used: R0,R1,R16,R17,XL,XH

RTC_GetDate:	ldi	XL,Low(Date)
		ldi	XH,High(Date)

		in	R16,MCUCR
		push	R16
		andi	R16,0b01111111		;External SRAM Disabled
		out	MCUCR,R16

		sbi	RTC_CEPort,RTC_CE	;CE = H, IC active

		ldi	R16,'2'
		st	X+,R16
		ldi	R16,'0'
		st	X+,R16			;20yy-mm-dd


		ldi	R17,0xD			;year tens
		mov	R1,R17
		ldi	R17,3

RTC_GetD1:	mov	R16,R1
		dec	R1
		rcall	RTC_BurstRD
		ori	R16,0x30
		st	X+,R16
		mov	R16,R1
		dec	R1
		rcall	RTC_BurstRD
		ori	R16,0x30
		st	X+,R16
		ldi	R16,'-'
		st	X+,R16
		dec	R17
		brne	RTC_GetD1

		cbi	RTC_CEPort,RTC_CE	;CE = L, IC inactive
		nop
		cbi	RTC_SIOPort,RTC_SIO
		sbi	RTC_SIODIR,RTC_SIO	;RTC_SIO Output = L

		pop	R0
		out	MCUCR,R0		;MCUCR visszaallitasa
		ret


;***************************************************************************
; RTC Read Data
;
; In:  R16 address
; Out: R16 data (low four bits)
; Used: R0

RTC_ReadData:
		andi	R16,0b00001111		;csak az also 4 bit kell (cim)
		ori	R16,0b01100000		;R/W & AD = 1 DT=0, read mode
		mov	R0,R16

		in	R16,MCUCR
		push	R16
		andi	R16,0b01111111		;External SRAM Disabled
		out	MCUCR,R16

		sbi	RTC_CEPort,RTC_CE	;CE = H, IC active

		rcall	RTC_SetAD
		rcall	RTC_GetDat
						;R0-ban a beolvasott ertek
		cbi	RTC_CEPort,RTC_CE	;CE = L, IC inactive
		ldi	R16,0x0F
		and	R16,R0			;csak az also 4 bit kell
		cbi	RTC_SIOPort,RTC_SIO
		sbi	RTC_SIODIR,RTC_SIO	;RTC_SIO Output = L

		pop	R0
		out	MCUCR,R0		;MCUCR visszaallitasa
		ret




;***************************************************************************
; RTC Burst Read Data
;
; In:  R16 address
; Out: R16 data (low four bits)
; Used: R0

RTC_BurstRD:
		andi	R16,0b00001111		;csak az also 4 bit kell (cim)
		ori	R16,0b01100000		;R/W & AD = 1 DT=0, read mode
		mov	R0,R16
		rcall	RTC_SetAD
		rcall	RTC_GetDat		;R0-ban a beolvasott ertek
		sbi	RTC_CLKPort,RTC_CLK	;CLK = H
						;bus dir change!
		ldi	R16,0x0F
		and	R16,R0			;csak az also 4 bit kell
		cbi	RTC_SIOPort,RTC_SIO
		sbi	RTC_SIODIR,RTC_SIO	;RTC_SIO Output = L
		ret



;***************************************************************************
; RTC Write Data
;
; In:  R16 address
;      R17 data
; Out: -
; Used: R0

RTC_WriteData:
		andi	R16,0b00001111		;csak az also 4 bit kell (cim)
		ori	R16,0b00100000		;AD = 1, RW & DT=0, write mode
		mov	R0,R16

		in	R16,MCUCR
		push	R16
		andi	R16,0b01111111		;External SRAM Disabled
		out	MCUCR,R16

		sbi	RTC_CEPort,RTC_CE	;CE = H, IC active

		rcall	RTC_SetAD

		andi	R17,0b00001111		;csak az also 4 bit kell (cim)
		ori	R17,0b00010000		;AD = 1, RW & DT=0, write mode
		mov	R0,R17
		rcall	RTC_SetAD

		nop
		cbi	RTC_CEPort,RTC_CE	;CE = L, IC inactive

		pop	R0
		out	MCUCR,R0		;MCUCR visszaallitasa
		ret




;***************************************************************************
; RTC Burst Write Data
;
; In:  R16 address
;      R17 data
; Out: -
; Used: R0

RTC_BurstWD:
		andi	R16,0b00001111		;csak az also 4 bit kell (cim)
		ori	R16,0b00100000		;AD = 1, RW & DT=0, write mode
		mov	R0,R16

		rcall	RTC_SetAD

		andi	R17,0b00001111		;csak az also 4 bit kell (cim)
		ori	R17,0b00010000		;AD = 1, RW & DT=0, write mode
		mov	R0,R17
		rcall	RTC_SetAD

		ret


;***********************************************
;* Subrutin for RTC_ReadData & RTC_WriteData
;*

RTC_SetAD:	ldi	R16,8			;CE setup time min 300ns
						;@8MHz 4 cycles
		
RTC_Rd0:	rol	R0			;MSB to carry
		brcc	RTC_bitnull
		sbi	RTC_SIOPort,RTC_SIO	;SIO = 1
		rjmp	RTC_bitok

RTC_bitnull:	cbi	RTC_SIOPort,RTC_SIO	;SIO = 0

RTC_bitok:	nop
		nop
		sbi	RTC_CLKPort,RTC_CLK	;CLK = H
		nop				;min 300ns ki kell tartani
		nop				;@8MHz => 4 cycles
		nop
		nop
		cbi	RTC_CLKPort,RTC_CLK	;CLK = L
		nop
		nop
		dec	R16
		brne	RTC_Rd0			;R1>0 vissza az elejere		
		ret


;***********************************************
RTC_GetDat:					;Dumy cycle & SIO direction change
		cbi	RTC_SIODIR,RTC_SIO	;RTC_SIO Input, Pull-up
		sbi	RTC_SIOPort,RTC_SIO
		nop
		nop

		sbi	RTC_CLKPort,RTC_CLK	;CLK = H
		nop				;min 300ns ki kell tartani
		nop				;@8MHz => 4 cycles
		nop
		nop
		nop
		cbi	RTC_CLKPort,RTC_CLK	;CLK = L
		nop
		ldi	R16,7			;end of dumy cycle
						;read 7 bit from IC
	
RTC_Rd1:	nop
		nop	
		sbi	RTC_CLKPort,RTC_CLK	;CLK = H
		nop				;min 300ns ki kell tartani
		nop				;@8MHz => 4 cycles
		nop
		nop
		clc				;carry = L
		sbic	RTC_SIOPIN,RTC_SIO	;a kovetkezo kimarad ha SIO=L
		 sec				;carry = H
		rol	R0			;beolvasott bit berotal
		cbi	RTC_CLKPort,RTC_CLK	;CLK = L
		nop
		dec	R16
		brne	RTC_Rd1			;R16>0 vissza az elejere		
        	ret